<?php

namespace common\services;

use common\helpers\MeanPayment;
use common\models\CreditHistory;
use common\models\Order;
use yii\helpers\Html;

class CreditHistoryService
{
    /**
     * Enregistre le modèle.
     *
     * @param boolean $runValidation
     * @param array $attributeNames
     */
    public function save($creditHistory)
    {
        if ($creditHistory->getAmount() > -0.01 && $creditHistory->getAmount() < 0.01) {
            return false;
        }

        // initialisation du commentaire avant sauvegarde
        $creditHistory->setComment($creditHistory->getComment() . $this->getStrComment($creditHistory));

        $creditHistory->save();

        // Mise à jour du crédit au niveau de UserProducer
        Yii::$app->logic->getUserProducerContainer()->getService()->updateCredit($creditHistory);
    }

    /**
     * Retourne si le CreditHistorique est un débit ou non.
     *
     * @return boolean
     */
    public function isTypeDebit($creditHistory)
    {
        return in_array($creditHistory->type, [
            CreditHistory::TYPE_DEBIT,
            CreditHistory::TYPE_PAYMENT,
        ]);
    }

    /**
     * Retourne si le CreditHistorique est un crédit ou non.
     *
     * @return boolean
     */
    public function isTypeCredit($creditHistory)
    {
        return in_array($creditHistory->type, [
            CreditHistory::TYPE_CREDIT,
            CreditHistory::TYPE_INITIAL_CREDIT,
            CreditHistory::TYPE_REFUND
        ]);
    }

    /**
     * Retourne le montant.
     *
     * @param boolean $format
     * @return float
     */
    public function getAmount($creditHistory, $format = false)
    {
        if ($format) {
            return number_format($creditHistory->amount, 2) . '&nbsp;€';
        }
        else {
            return $creditHistory->amount;
        }
    }

    /**
     * Retourne le libellé du CreditHistory informant de son type et
     * éventuellement de la date de sa commande associée.
     *
     * @return string
     */
    public function getStrWording($creditHistory)
    {
        $str = '';

        if ($creditHistory->type == CreditHistory::TYPE_INITIAL_CREDIT) {
            $str = 'Crédit initial';
        }
        elseif ($creditHistory->type == CreditHistory::TYPE_CREDIT) {
            $str = 'Crédit';
        }
        elseif ($creditHistory->type == CreditHistory::TYPE_PAYMENT) {
            $str = 'Paiement';
        }
        elseif ($creditHistory->type == CreditHistory::TYPE_REFUND) {
            $str = 'Remboursement';
        }
        elseif ($creditHistory->type == CreditHistory::TYPE_DEBIT) {
            $str = 'Débit';
        }

        if ($creditHistory->type == CreditHistory::TYPE_PAYMENT || $creditHistory->type == CreditHistory::TYPE_REFUND) {
            if (isset($creditHistory->order) && isset($creditHistory->order->distribution)) {
                $str .= '<br />Commande : ' . date('d/m/Y', strtotime($creditHistory->order->distribution->date));
            }
            else {
                $str .= '<br />Commande supprimée';
            }
        }

        return $str;
    }

    /**
     * Retourne les informations à ajouter au commentaire du CreditHistorique
     * (libellé, montant, client, action) au format HTML.
     *
     * @return string
     */
    public function getStrComment($creditHistory)
    {
        $str = '';
        if (strlen($creditHistory->getComment())) {
            $str .= '<br />';
        }

        $str .= $this->getStrWording();
        if ($creditHistory->getOrder()) {
            $str .= '<br />Montant de la commande : ' . $creditHistory->getOrder()->getAmountWithTax(Order::AMOUNT_TOTAL, true);
        }

        if ($creditHistory->getUser()) {
            $str .= '<br />Client : ' . Html::encode($creditHistory->getUser()->name . ' ' . $creditHistory->user->lastname);
        }

        if ($creditHistory->getUserAction()) {
            $str .= '<br />Action : ' . Html::encode($creditHistory->getUserAction()->name . ' ' . $creditHistory->userAction->lastname);
        }

        return $str;
    }

    /**
     * Retourne la date.
     *
     * @param boolean $format
     * @return string
     */
    public function getDate($creditHistory, $format = false)
    {
        $date = $creditHistory->getDate();

        if ($format) {
            return date('d/m/Y à H:i:s', strtotime($date));
        }
        else {
            return $date;
        }
    }

    /**
     * Retourne le libellé du moyen de paiement du CreditHistory courant.
     *
     * @return string
     */
    public function getStrMeanPayment($creditHistory)
    {
        return MeanPayment::getStrBy($creditHistory->getMeanPayment());
    }

    /**
     * Retourne le libellé de l'utilisateur ayant initié l'action.
     *
     * @return string
     */
    // strUserAction
    public function getStrUserAction($creditHistory)
    {
        $userAction = $creditHistory->getUserActionObject();

        if ($userAction) {
            return $userAction->name . ' ' . $userAction->lastname;
        }
        else {
            return 'Système';
        }
    }
}