<?php

/**
 * Copyright distrib (2018)
 *
 * contact@opendistrib.net
 *
 * Ce logiciel est un programme informatique servant à aider les producteurs
 * à distribuer leur production en circuits courts.
 *
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 */

namespace common\models;

use common\components\ActiveRecordCommon;

/**
 * This is the model class for table "credit_historique".
 *
 * @property integer $id
 * @property integer $id_user
 * @property integer $id_order
 * @property string $date
 * @property double $amount
 * @property string $type
 * @property integer $id_producer
 * @property string $mean_payment
 */
class CreditHistory extends ActiveRecordCommon
{
    const TYPE_INITIAL_CREDIT = 'initial-credit';
    const TYPE_CREDIT = 'credit';
    const TYPE_PAYMENT = 'payment';
    const TYPE_REFUND = 'refund';
    const TYPE_DEBIT = 'debit';

    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'credit_history';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['amount'], 'required'],
            [['id_user', 'id_user_action', 'id_order', 'id_producer'], 'integer'],
            [['date'], 'safe'],
            [['amount'], 'double'],
            [['type', 'mean_payment'], 'string', 'max' => 255],
            [['comment'], 'string', 'max' => 2048],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'id_user' => 'Utilisateur',
            'id_user_action' => 'Utilisateur',
            'id_order' => 'Commande',
            'date' => 'Date',
            'amount' => 'Montant',
            'type' => 'Type',
            'id_producer' => 'Producteur',
            'mean_payment' => 'Moyen de paiement',
            'comment' => 'Commentaire',
        ];
    }

    /*
     * Relations
     */

    public function getUser()
    {
        return $this->hasOne(User::className(), ['id' => 'id_user']);
    }

    public function getUserObject()
    {
        return $this->user;
    }

    public function getUserAction()
    {
        return $this->hasOne(User::className(), ['id' => 'id_user_action']);
    }

    public function getUserActionObject()
    {
        return $this->userAction;
    }

    public function getOrder()
    {
        return $this->hasOne(Order::className(), ['id' => 'id_order']);
    }

    public function getOrderObject()
    {
        return $this->order;
    }


    /*
     * Getters / setters
     */

    public function getId()
    {
        return $this->id;
    }

    public function getIdUser()
    {
        return $this->id_user;
    }

    public function setIdUser($idUser)
    {
        $this->id_user = $idUser;

        return $this;
    }

    public function getIdUserAction()
    {
        return $this->id_user_action;
    }

    public function setIdUserAction($idUserAction)
    {
        $this->id_user_action = $idUserAction;

        return $this;
    }

    public function getIdOrder()
    {
        return $this->id_order;
    }

    public function setIdOrder($idOrder)
    {
        $this->id_order = $idOrder;

        return $this;
    }

    public function getDate()
    {
        return $this->date;
    }

    public function setDate($date)
    {
        $this->date = $date;

        return $this;
    }

    public function getAmount()
    {
        return $this->amount;
    }

    public function setAmount($amount)
    {
        $this->amount = $amount;

        return $this;
    }

    public function getType()
    {
        return $this->type;
    }

    public function setType($type)
    {
        $this->type = $type;

        return $this;
    }

    public function getIdProducer()
    {
        return $this->id_producer;
    }

    public function setIdProducer($idProducer)
    {
        $this->id_producer = $idProducer;

        return $this;
    }

    public function getMeanPayment()
    {
        return $this->mean_payment;
    }

    public function setMeanPayment($meanPayment)
    {
        $this->mean_payment = $meanPayment;

        return $this;
    }

    public function getComment()
    {
        return $this->comment;
    }

    public function setComment($comment)
    {
        $this->comment = $comment;

        return $this;
    }
}
